/*
 * Photo Image Print System
 * Copyright (C) 2001-2005 EPSON AVASYS Corporation.
 * Copyright (C) Seiko Epson Corporation 2001-2008.
 *
 * This file is part of the `espmntg' program.
 *
 *  This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
 
#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif
#include <gtk/gtk.h>
#include <string.h>
#include <unistd.h>
#include <sys/wait.h>

#include "support.h"
#include "interface.h"
#include "dlgstrings.h"
#include "funcMan.h"

#ifdef DEBUG
#include "ekpdbg.h"
#endif
#ifndef DP
#define DP(e)
#endif


#define DIR_PIXMAP	PIPS_GUI_DIR"/pixmaps/"
#define CMDPATH		"espmnt"

enum IV_COLMUN{
	eIVC_ID, eIVC_LABEL, eIVC_IMAGE
};

enum WND_ID{
	eWID_NULL,
	eWID_NOZZLE1, eWID_NOZZLE2,
	eWID_CLEAN1, eWID_CLEAN2,
	eWID_GAP1, eWID_GAP2, eWID_GAP3,
	eWID_GAP4, eWID_GAP5, eWID_GAP6, eWID_GAP7
};

#define WPNAME_ID	"WINDID"
#define CTLNAME_PRMBTN	"btnExec"
#define CTLNAME_SCDBTN	"btnExec2"
#define CTLNAME_FINBTN	"btnFin"
#define CTLNAME_DSCLBL	"lblDesc"

/* Common Window IF */
typedef GtkWidget* (*wndStartup)(void);
typedef int        (*wndStartupExt)(GtkWidget* wnd, int wndID);
typedef int        (*wndSetParam)(char *pCmdBuff, GtkWidget* wnd);

/* window structure */
typedef struct {
	int		wndID;
	char	iconName[32];
	char	pxm[128];
}EU_FUNC_ENT;

typedef struct {
	char		cmd[32];
	int			nextWnd;
}EU_BUTTON_ACT;

typedef struct {
	int				id;
	wndStartup		startup;
	wndStartupExt	startupExt;
	wndSetParam		setparam;
	char			*description;
	char			keycmd[32];
	EU_BUTTON_ACT	btn1;
	EU_BUTTON_ACT	btn2;	
}EU_FUNC_WNDENT;


static GtkWidget *CreateWnd(int wndID);
static const EU_FUNC_WNDENT* GetWndEntry(int wndID);
static void LockWindow(GtkWidget* wnd, gboolean lock);
static int ExecCommandLine(const char *cmd, const char *opt, const char *prm, gchar **pStdout, gchar **pStdErr);
void OnbtnExec(GtkButton *button, gpointer user_data);
gboolean OnDeleteEvent(GtkWidget *widget, GdkEvent *event, gpointer user_data);

static int CreateGap3Wnd(GtkWidget* wnd, int wndID);
static int SetGapParam(char *pCmdBuff, GtkWidget* wnd);


static EU_FUNC_ENT c_funcmap[] = {
	{eWID_NOZZLE1,	N_("Nozzle Check"),			DIR_PIXMAP"viNozzle.xpm"}, 
	{eWID_CLEAN1,	N_("Head Cleaning"),		DIR_PIXMAP"viClean.xpm"	}, 
	{eWID_GAP1,		N_("Print Head Alignment"),	DIR_PIXMAP"viGap.xpm"	}, 
};
#define EU_FUNC_COUNT sizeof(c_funcmap)/sizeof(EU_FUNC_ENT)

static EU_FUNC_WNDENT c_wndmap[] = {
	{eWID_NOZZLE1,	create_wndNozzle1,	NULL, 			NULL,		EUSTR_NOZZLE1,	"n",
		{"n\0\0", eWID_NOZZLE2},	{"\0\0", eWID_NULL} }, 
	{eWID_NOZZLE2,	create_wndNozzle2,	NULL, 			NULL,		EUSTR_NOZZLE2,	"n",	
		{"\0\0",  eWID_CLEAN1},		{"\0\0", eWID_NULL} }, 
	{eWID_CLEAN1,	create_wndClean1,	NULL, 			NULL,		EUSTR_CLEAN1,	"c",
		{"c\0\0", eWID_CLEAN2},		{"\0\0", eWID_NULL} }, 
	{eWID_CLEAN2,	create_wndClean2,	NULL, 			NULL,		EUSTR_CLEAN2,	"c",
	 	{"\0\0",  eWID_NOZZLE1},	{"\0\0", eWID_NULL} }, 	
	{eWID_GAP1,		create_wndGap1,		NULL, 			NULL,		EUSTR_GAP1,		"gp",
	 	{"\0\0",  eWID_GAP2},		{"\0\0", eWID_NULL} }, 	
	{eWID_GAP2,		create_wndGap2,		NULL, 			NULL,		EUSTR_GAP2,		"gp",
		{"gp\0\0",eWID_GAP3},		{"\0\0", eWID_GAP4} }, 	
	{eWID_GAP3,		create_wndGap3,		CreateGap3Wnd,	SetGapParam,EUSTR_GAP3,		"gs",
		{"gs\0\0", eWID_GAP2},		{"gs\0\0", eWID_GAP4} }, 	
	{eWID_GAP4,		create_wndGap2,		NULL,			NULL,		EUSTR_GAP4,		"gpb",
		{"gpb\0\0",   eWID_GAP5},	{"\0\0", eWID_GAP6} }, 	
	{eWID_GAP5,		create_wndGap3,		CreateGap3Wnd,	SetGapParam,EUSTR_GAP5,		"gsb",
		{"gsb\0\0",eWID_GAP4},		{"gsb\0\0",eWID_GAP6} }, 	
	{eWID_GAP6,		create_wndGap2,		NULL,			NULL,		EUSTR_GAP6,		"gpc",
		{"gpc\0\0",	  eWID_GAP7},	{"\0\0", eWID_NULL} }, 	
	{eWID_GAP7,		create_wndGap3,		CreateGap3Wnd,	SetGapParam,EUSTR_GAP7,		"gsc",
		{"gsc\0\0",eWID_GAP6},		{"gsc\0\0",eWID_NULL} }, 	
};
#define EU_WND_COUNT sizeof(c_wndmap)/sizeof(EU_FUNC_WNDENT)

static char 	 m_printer[128];
static gboolean  m_cmddoing = FALSE;
static gint		 m_cmdstat = 0;


void childWatchFunc(GPid pid,
                   gint status,
                   gpointer data)
{
DP(("childWatchFunc\n"));
	m_cmddoing = FALSE;
	if( WIFEXITED(status) && WEXITSTATUS(status) == 0){
		m_cmdstat = NO_ERROR;
	} else {
		m_cmdstat = COMMAND_ERROR;
	} 
//	g_spawn_close_pid(pid);
}


int ExecCommandLine(const char *cmd, const char *opt, const char *prm, gchar **pStdout, gchar **pStderr)
{
#define PRM_NUM		6
	int 	res = NO_ERROR;
	int n;
	gchar *args[PRM_NUM];
	GPid child_pid;
	gint fdOut, fdErr;
 	GIOChannel *chOut, *chErr;
	GError	*pgeRes = NULL;
	
	memset(args, 0, sizeof(args));
	n = 0;
	args[n++] = g_strdup(CMDPATH);
	args[n++] = g_strdup("--int");
	if(opt && *opt){
		args[n++] = g_strdup_printf("-%s%s", cmd, opt);
	} else{
		args[n++] = g_strdup_printf("-%s", cmd);
	}
	if( strlen(m_printer) ){
		args[n++] = g_strdup_printf("-p%s", m_printer);
	}
	if(prm && *prm){
		args[n++] = g_strdup_printf("-v%s", prm);
	}
	args[n] = NULL;

	if( g_spawn_async_with_pipes(NULL, &args[0], NULL, 
									G_SPAWN_SEARCH_PATH | G_SPAWN_DO_NOT_REAP_CHILD,
									NULL, NULL, &child_pid,
									NULL, &fdOut, &fdErr, &pgeRes) )
	{
DP(("child_pid = %d", child_pid));
		m_cmddoing = TRUE;
		m_cmdstat = 0;
		g_child_watch_add(child_pid, childWatchFunc, NULL);
		
 		chOut = g_io_channel_unix_new (fdOut);
 		chErr = g_io_channel_unix_new (fdErr);

		while( m_cmddoing ){
			while (gtk_events_pending()){
				gtk_main_iteration();
			}
			  
			sleep(1);
		}
DP((" ...  END\n"));

 		g_io_channel_read_to_end(chOut, pStdout, NULL, NULL);
 		g_io_channel_shutdown (chOut, FALSE, NULL);
 		g_io_channel_unref (chOut);

		if( 0!= m_cmdstat){
	 		g_io_channel_read_to_end(chErr, pStderr, NULL, NULL);
	 	}
 		g_io_channel_shutdown (chErr, FALSE, NULL);
 		g_io_channel_unref (chErr);

		g_spawn_close_pid(child_pid);
DP(("OUT : %s//\n", *pStdout));
DP(("ERR : %s//\n", *pStderr));
DP(("RET : %d\n", m_cmdstat));
		res = m_cmdstat;
		
	} else{
		if(*pStderr){
			g_free(*pStderr);
			*pStderr = NULL;
		}
		*pStderr = g_strdup(pgeRes->message);
		res = COMMAND_ERROR;
	}	

	for(n = 0; n < PRM_NUM; n++){
		if(args[n]){
			g_free(args[n]);
		}
	}

	return res;
}


void OnbtnExec(GtkButton *button, gpointer user_data)
{
	EU_BUTTON_ACT  *act = (EU_BUTTON_ACT*)user_data;
	const EU_FUNC_WNDENT *wndEnt = NULL;
	int 	wndID;

	char	*pWndCmd;
	char	prm[256];
	gchar	*pStdout = NULL;
	gchar	*pStderr = NULL;
	int 	res = NO_ERROR;
	
	GtkWidget* wndCur;
	GtkWidget* wnd;
	
	wndCur = GetParentWindow(GTK_WIDGET(button));
	wndID = (gint)g_object_get_data(G_OBJECT(wndCur), WPNAME_ID);
	wndEnt = GetWndEntry(wndID);

	gdk_window_set_cursor (wndCur->window, gdk_cursor_new (GDK_WATCH));
	LockWindow(wndCur, TRUE);

	pWndCmd = act->cmd;
	while(*pWndCmd){
		/* make parameter string */
		prm[0] = '\0';
		if( wndEnt->setparam ){
			wndEnt->setparam(prm, wndCur);
		}
		
		/* Exec Command */
		res = ExecCommandLine("e", pWndCmd, prm, &pStdout, &pStderr);
		if(pStdout){
			g_free(pStdout);
		}
		if(NO_ERROR != res){
			break;
		}

		pWndCmd += strlen(pWndCmd)+1;
	}

	LockWindow(wndCur, FALSE);
	gdk_window_set_cursor (wndCur->window, gdk_cursor_new (GDK_ARROW));
	
	if(NO_ERROR == res){
		/* Show Next Window */
		wnd = CreateWnd(act->nextWnd);
		if(wnd){
			gtk_widget_show_now (wnd);
		}
		/* delete current window */
		gtk_widget_destroy( wndCur );
	} else{
		/* Show error message */
		GtkWidget *dlgErr;		
		dlgErr = gtk_message_dialog_new (GTK_WINDOW(wndCur) ,
									GTK_DIALOG_MODAL |
									GTK_DIALOG_DESTROY_WITH_PARENT ,
									GTK_MESSAGE_ERROR,
									GTK_BUTTONS_OK,
									pStderr);
		gtk_dialog_run (GTK_DIALOG(dlgErr));
		gtk_widget_destroy (dlgErr);
	}
	
}

gboolean OnDeleteEvent(GtkWidget *widget, GdkEvent *event, gpointer user_data)
{
DP(("on_delete_event\n"));
	if( m_cmddoing ){
		return TRUE;
	} else{
		return FALSE;
	}
}

                                        
int SetupFunctions(const char *printer)
{
	gchar	*pStdout = NULL;
	gchar	*pStderr = NULL;

	int 	res = NO_ERROR;
	gint	nIdx, nCnt;
	int		avail = 0;

	m_printer[0] = '\0';
	
	if( printer && sizeof(m_printer) > strlen(printer) ){
		strcpy(m_printer, printer);
	}

	/* Get parameter info */
	res = ExecCommandLine("f", NULL, NULL, &pStdout, &pStderr);
	if(pStderr){
		fprintf(stderr, "%s\n", pStderr);
		g_free(pStderr);
	}
DP(("A %d %s\n", res, pStdout));
	if( res != NO_ERROR || !pStdout ){
		if(pStdout)g_free(pStdout);
		return COMMAND_ERROR;
	}
	/* Remove unsupported function window form window map */
	for(nIdx = 0; nIdx < EU_WND_COUNT; nIdx++){
		avail = 0;
		if(strstr(pStdout, c_wndmap[nIdx].keycmd)){
			avail = 1;
		}
				
		if( !avail ){
			/* disconnect window refarence */
			for(nCnt = 0; nCnt < EU_WND_COUNT; nCnt++){	
				if( c_wndmap[nIdx].id == c_wndmap[nCnt].btn1.nextWnd ){
					c_wndmap[nCnt].btn1.nextWnd = eWID_NULL;
				}
				if( c_wndmap[nIdx].id == c_wndmap[nCnt].btn2.nextWnd ){
					c_wndmap[nCnt].btn2.nextWnd = eWID_NULL;
				}
			}
		
			c_wndmap[nIdx].id = eWID_NULL;
		}
	}
	
	/* Remove unsupported function form function map */
	for(nIdx = 0; nIdx < EU_FUNC_COUNT; nIdx++){	
		for(nCnt = 0; nCnt < EU_WND_COUNT; nCnt++){	
			if(c_wndmap[nCnt].id == c_funcmap[nIdx].wndID){
				break;
			}
		}
		if(nCnt >= EU_WND_COUNT){
			c_funcmap[nIdx].wndID = eWID_NULL;
		}
	}	

	g_free(pStdout);
	return 0;
}


void AppendIcons(GtkIconView *pView)
{
	GtkListStore	*pStore;
	GtkTreeIter		it;
	
	GdkPixbuf		*pPix;
	int 			n = 0;
	
	pStore = gtk_list_store_new(3, G_TYPE_INT, G_TYPE_STRING, GDK_TYPE_PIXBUF);
	
	gtk_icon_view_set_model(pView, GTK_TREE_MODEL(pStore));
	gtk_icon_view_set_text_column(pView, eIVC_LABEL);
	gtk_icon_view_set_pixbuf_column(pView, eIVC_IMAGE);

	/* Add functions icon */
	for(; n < EU_FUNC_COUNT; n++){
		if( eWID_NULL != c_funcmap[n].wndID ){
			pPix = gdk_pixbuf_new_from_file(c_funcmap[n].pxm, NULL);
			if(NULL == pPix){
				DP(("image not found : %s\n", c_funcmap[n].pxm));
			}
			gtk_list_store_append(pStore, &it);
			gtk_list_store_set(pStore, &it, 
							eIVC_ID,	c_funcmap[n].wndID,
							eIVC_LABEL, _(c_funcmap[n].iconName),
							eIVC_IMAGE, pPix,
							-1);
		}
	}
}


void RunFunctionWnd(GtkIconView *iconview, GtkTreePath *path)
{
	GtkTreeModel	*model;
	GtkTreeIter		it;
	int				wndID = -1;

	GtkWidget* wnd = NULL;

	/* Get Icon label */	
	model = gtk_icon_view_get_model(iconview);
	if(model && gtk_tree_model_get_iter(model, &it, path)){
		gtk_tree_model_get(model, &it, eIVC_ID, &wndID, -1);
	} else{
		DP(("gtk_tree_model_get fail\n"));
	}
	
	wnd = CreateWnd(wndID);
	if(wnd){
		gtk_widget_show (wnd);
	}	
}


void LockWindow(GtkWidget* wnd, gboolean lock)
{
	GtkWidget *btn;
	if(lock){
		/* Enable buttons */
		btn = lookup_widget(wnd, CTLNAME_PRMBTN);
		if(btn){
			g_signal_handlers_block_matched ((gpointer) btn, G_SIGNAL_MATCH_FUNC,
		                                 0, g_quark_from_string("clicked"), NULL,
		                                 G_CALLBACK (OnbtnExec),
		                                 NULL);	
		}

		btn = lookup_widget(wnd, CTLNAME_SCDBTN);
		if(btn){
			g_signal_handlers_block_matched ((gpointer) btn, G_SIGNAL_MATCH_FUNC,
		                                 0, g_quark_from_string("clicked"), NULL,
		                                 G_CALLBACK (OnbtnExec),
		                                 NULL);	
		}

		btn = lookup_widget(wnd, CTLNAME_FINBTN);
		if(btn){
			g_signal_handlers_block_matched ((gpointer) btn, G_SIGNAL_MATCH_FUNC,
		                                 0, g_quark_from_string("clicked"), NULL,
		                                 G_CALLBACK (gtk_widget_destroy),
		                                 NULL);	
		}
	} else{
		/* Disable buttons */
		btn = lookup_widget(wnd, CTLNAME_PRMBTN);
		if(btn){
			g_signal_handlers_unblock_matched ((gpointer) btn, G_SIGNAL_MATCH_FUNC,
		                                 0, g_quark_from_string("clicked"), NULL,
		                                 G_CALLBACK (OnbtnExec),
		                                 NULL);	
		}

		btn = lookup_widget(wnd, CTLNAME_SCDBTN);
		if(btn){
			g_signal_handlers_unblock_matched ((gpointer) btn, G_SIGNAL_MATCH_FUNC,
		                                 0, g_quark_from_string("clicked"), NULL,
		                                 G_CALLBACK (OnbtnExec),
		                                 NULL);	
		}

		btn = lookup_widget(wnd, CTLNAME_FINBTN);
		if(btn){
			g_signal_handlers_unblock_matched ((gpointer) btn, G_SIGNAL_MATCH_FUNC,
		                                 0, g_quark_from_string("clicked"), NULL,
		                                 G_CALLBACK (gtk_widget_destroy),
		                                 NULL);	
		}
	}
}


GtkWidget *CreateWnd(int wndID)
{
	int 		nIdx = 0;
	GtkWidget* wnd = NULL;
	GtkWidget* button;
	GtkWidget* label;
	
	if( eWID_NULL == wndID ){
		return NULL;
	}
	
DP(("CreateWnd (%d/%d)\n", wndID, EU_WND_COUNT));
	for(; nIdx < EU_WND_COUNT; nIdx++){	
		if( c_wndmap[nIdx].id == wndID ){
			wnd = c_wndmap[nIdx].startup();
			break;
		}
	}
	
	if(wnd){
		if(c_wndmap[nIdx].startupExt){
			c_wndmap[nIdx].startupExt(wnd, wndID);
		}
		
		/* set description message */
		label = lookup_widget(wnd, CTLNAME_DSCLBL);
		if( label ){
			gtk_label_set_text(GTK_LABEL(label), _(c_wndmap[nIdx].description));
		}

		/* wndid refer to window  */
		g_object_set_data(G_OBJECT(wnd), WPNAME_ID, (gpointer)wndID);

		/* set execute button action */
		if( strlen(c_wndmap[nIdx].btn1.cmd) || 
			c_wndmap[nIdx].btn1.nextWnd != eWID_NULL){		/* primary button */
			
			button = lookup_widget(wnd, CTLNAME_PRMBTN);
			g_signal_connect ((gpointer) button, "clicked",
                            G_CALLBACK (OnbtnExec),
                            (gpointer)&(c_wndmap[nIdx].btn1));
    	}

		button = lookup_widget(wnd, CTLNAME_SCDBTN);
		if( button ){
			if( strlen(c_wndmap[nIdx].btn2.cmd) || 
			  	c_wndmap[nIdx].btn2.nextWnd != eWID_NULL ){	/* secondary button */
			  	DP(("button2 %s : %d\n", c_wndmap[nIdx].btn2.cmd, c_wndmap[nIdx].btn2.nextWnd));
			
				g_signal_connect ((gpointer) button, "clicked",
                            G_CALLBACK (OnbtnExec),
                            (gpointer)&(c_wndmap[nIdx].btn2));
			} else{
				/* hide uneffective bottun */
				gtk_widget_hide(button);
			}
		}
		
		g_signal_connect( (gpointer) wnd, "delete_event",
						G_CALLBACK (OnDeleteEvent), NULL );
	} else{
		DP(("Target window not found\n"));
	}
                            	
	return wnd;
}


const EU_FUNC_WNDENT* GetWndEntry(int wndID)
{ 
	int nIdx = 0;
	for(; nIdx < EU_WND_COUNT; nIdx++){	
		if( c_wndmap[nIdx].id == wndID){
			return &c_wndmap[nIdx];
			break;
		}
	}
	
	return NULL;
}


GtkWidget *GetParentWindow(GtkWidget *ctrl)
{
	GtkWidget *parent = NULL;
	while(ctrl){
		ctrl = gtk_widget_get_parent(ctrl);
		if(ctrl){
			parent = ctrl;
		}
	}
	
	return parent;
}

/* ---------------------------------------------------------------------
	Window depend functions 
--------------------------------------------------------------------- */

int CreateGap3Wnd(GtkWidget* wnd, int wndID)
{
	const EU_FUNC_WNDENT*	wndent = NULL;
	
	GtkWidget *vboxGapPrm;
	GtkWidget *hboxGapPrm;
	GtkWidget *lblGapPrm;
	GtkWidget *cmbGapPrm;
	GtkWidget* button;
	
	int 	res = NO_ERROR;
	gchar	*pStdout = NULL;
	gchar	*pStderr = NULL;

	gint	nIdx, nCnt;
	int		nMin, nMax;
	char	*pPos = NULL;
	char	strLabel[32];

	wndent = GetWndEntry(wndID);
	if( !wndent ){
		return LIB_INTERNAL_ERROR;
	}

	/* Get parameter info */
	res = ExecCommandLine("n", wndent->btn1.cmd, NULL, &pStdout, &pStderr);
	if(pStderr)g_free(pStderr);
	if( res != NO_ERROR || !pStdout ){
		if(pStdout)g_free(pStdout);
		return COMMAND_ERROR;
	}
	
	/* Create parameter combos */
	vboxGapPrm = lookup_widget(wnd, "vboxGapPrm");
	
	pPos = (char*)strtok(pStdout, ",");
	nIdx = 1;
	while(pPos){
		hboxGapPrm = gtk_hbox_new (FALSE, 8);
		gtk_widget_show (hboxGapPrm);
		gtk_box_pack_start (GTK_BOX (vboxGapPrm), hboxGapPrm, TRUE, TRUE, 4);

		sprintf(strLabel, "#%d", nIdx);
		lblGapPrm = gtk_label_new (strLabel);
		gtk_widget_show (lblGapPrm);
		gtk_box_pack_start (GTK_BOX (hboxGapPrm), lblGapPrm, FALSE, FALSE, 0);

		cmbGapPrm = gtk_combo_box_new_text ();
		gtk_widget_show (cmbGapPrm);
		gtk_box_pack_start (GTK_BOX (hboxGapPrm), cmbGapPrm, FALSE, FALSE, 0);
	
		sprintf(strLabel, "cmbGapPrm%d", nIdx);
		/*GLADE_HOOKUP_OBJECT (wnd, cmbGapPrm, strLabel);*/
		g_object_set_data_full(G_OBJECT(wnd), strLabel, gtk_widget_ref (cmbGapPrm), (GDestroyNotify) gtk_widget_unref);

		sscanf(pPos, "%d-%d", &nMin, &nMax);
DP(("min %d - max %d\n", nMin, nMax));
		for(nCnt = nMin; nCnt <= nMax; nCnt++){
			sprintf(strLabel, "%d", nCnt);
			gtk_combo_box_append_text (GTK_COMBO_BOX (cmbGapPrm), strLabel);
		}
		gtk_combo_box_set_active(GTK_COMBO_BOX (cmbGapPrm), (nMax-nMin)/2);
		
		nIdx++;
		pPos = (char*)strtok(NULL, ",");
	}
	g_free(pStdout);

	/* change secondary button label if it has next window */
	button = lookup_widget(wnd, CTLNAME_SCDBTN);
	if( button ){
DP(("second button %d\n", wndent->btn2.nextWnd));
		if( wndent->btn2.nextWnd != eWID_NULL ){
			/* gap alignment therd button labal */
			gtk_button_set_label(GTK_BUTTON(button), _("Next >"));
		}
	}
	
	return NO_ERROR;
}


int SetGapParam(char *pCmdBuff, GtkWidget* wnd)
{
	int 		nIdx = 1;
	char		ctrlName[32];
	GtkWidget*  ctrl;

	GtkTreeModel *model;
	GtkTreeIter iter;
	gchar 		*label;

	do{
		sprintf(ctrlName, "cmbGapPrm%d", nIdx);
		ctrl = lookup_widget(wnd, ctrlName);
		if(ctrl){
			model = gtk_combo_box_get_model( GTK_COMBO_BOX(ctrl) );
			if( gtk_combo_box_get_active_iter(GTK_COMBO_BOX(ctrl), &iter) ){
				gtk_tree_model_get(model , &iter , 0 , &label, -1);
				strcat(pCmdBuff, label);
				strcat(pCmdBuff, ",");
			} else{
DP(("invalid value select A\n"));
			}
		} else{
DP(("invalid value select B\n"));
		}
		
		nIdx++;
	}while(ctrl);

	return 0;
}



