/*
 * Photo Image Print System
 * Copyright (C) 2001-2005 EPSON AVASYS Corporation.
 * Copyright (C) Seiko Epson Corporation 2001-2008.
 *
 * This file is part of the `espmntg' program.
 *
 *  This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gtk/gtk.h>
#include <getopt.h>
#include <string.h>

#include "pipserr.h"
#include "setup.h"

#include "interface.h"
#include "support.h"
#include "funcMan.h"

#ifdef DEBUG
#include "ekpdbg.h"
#endif
#ifndef DP
#define DP(e)
#endif


int
main (int argc, char *argv[])
{
	GtkWidget *wndMain;
	GtkIconView *pView;
    char  printer[128];
    char* model_name = NULL;
    char* alias_name = NULL;
    char* pixmap_dir = NULL;
    char* usage = "Usage: -p [printer] or Not use argument for default printer.\n";
    int c;
    int index = 0;
    int res = NO_ERROR;
   struct option allopt[] =
        {
			{ "printer",1, 0, 'p'},
           {0,0,0,0}
        };

	gtk_set_locale ();
	gtk_init (&argc, &argv);

	printer[0] = '\0';
	
    while ((c = getopt_long (argc, argv, "p:", allopt, &index)) != -1)
    {
        switch (c)
        {
        case 'p':
            if (strlen (optarg) < sizeof(printer)-1){
                strcpy(printer, optarg);
            }
        	break;
        	
		default:
            if (!strcmp (argv[optind-1],"-p") && !optarg)
               FILTER_ERROR_HANDLER (OPTION_VALUES_ERROR, "-p");
            else
               FILTER_ERROR_HANDLER (INVALID_OPTION_ERROR, argv[1]);

            fprintf(stderr, usage);
            return 1;
        }
    }

	/* pritner name */
	if( strlen(printer) == 0 ){
		if( !(model_name = GetDefaultPrinter(1)) ){
			char **model_list = NULL;
			model_list = GetModelListName (PNG_TYPE);
			if( *model_list ){
				model_name = strdup(*model_list);
				free(model_list);
			}else {
				FILTER_ERROR_HANDLER (PRINTER_NOT_FOUND, "decide printer faile.");
				return PRINTER_NOT_FOUND;
			}
		}
	} else{
		model_name = strdup(printer);
	}

	alias_name = GetAliasName(model_name);
	if (alias_name == NULL) {
		alias_name = strdup(model_name);
	}
	if(model_name)free(model_name);
	

	/* text domain */
#ifdef ENABLE_NLS
    bindtextdomain (PACKAGE, LOCALEDIR);
    bind_textdomain_codeset (PACKAGE, "UTF-8");
    textdomain (PACKAGE);
    setlocale (LC_ALL, "");
#endif

	/* pixmap */
	add_pixmap_directory (PIPS_GUI_DIR"/pixmaps");
    pixmap_dir = (char*)malloc(strlen(PIPS_PRINTER_DIR)+strlen(alias_name)+12);
    if( !pixmap_dir ){
		FILTER_ERROR_HANDLER (MEMORY_ERROR, NULL);
		return MEMORY_ERROR;
    }
	sprintf(pixmap_dir, PIPS_PRINTER_DIR"/%s/pixmaps", alias_name);
    add_pixmap_directory(pixmap_dir);
    
	if( NO_ERROR != (res = SetupFunctions(printer)) ){
  		FILTER_ERROR_HANDLER (res, "configure faile.");
		return -1;
	}
   
	wndMain = create_wndMain ();
  
	/* Setup Icons */
	pView = (GtkIconView*)lookup_widget(wndMain, "ivwMain");
	if(NULL != pView){
		AppendIcons(pView);
	} else{
  		FILTER_ERROR_HANDLER (UNEXPECTED_ERROR, "main window setup faile.");
		return 1;
	}
    
	gtk_widget_show(wndMain);

	gtk_main();
 
	if(alias_name)free(alias_name);
	if(pixmap_dir)free(pixmap_dir);
  
  return 0;
}

